@@ -1,9 +1,6 @@
 #Vignettes that are computer intensive have been precompiled:
 
-library(knitr)
+library(knitr) #input, output
 knit("v4_simple_non_normal.Rmd.orig", "v4_simple_non_normal.Rmd")
 knit("v5_advanced_non_normal.Rmd.orig", "v5_advanced_non_normal.Rmd")
-knit("v6_expert_non_normal.Rmd.orig", "v6_expert_non_normal.Rmd")
-
-library(devtools)
-build_vignettes()
\ No newline at end of file
+knit("v6_expert_non_normal.Rmd.orig", "v6_expert_non_normal.Rmd")
\ No newline at end of file
